@@ -1,14 +1,14 @@
 #!perl
 # PODNAME: parse-syslog-line.pl
 # ABSTRACT: Parse a syslog message and display the structured data
-use v5.14;
+use v5.16;
 use warnings;
 
 use Data::Printer;
 use Getopt::Long::Descriptive;
 use Pod::Usage;
 use JSON::MaybeXS;
-use Parse::Syslog::Line qw( parse_syslog_line );
+use Parse::Syslog::Line;
 use YAML::XS;
 
 my $enc;
@@ -57,9 +57,9 @@ pod2usage(-verbose => 2, -exitval => 0 ) if $opt->manual;
 
 
 # Configure
+psl_enable_sdata() if $opt->sdata;
 $Parse::Syslog::Line::PruneEmpty = !$opt->empty;
 $Parse::Syslog::Line::PruneRaw   = !$opt->raw;
-$Parse::Syslog::Line::AutoDetectJSON = $Parse::Syslog::Line::AutoDetectKeyValues = $opt->sdata;
 $Parse::Syslog::Line::RFC5424StructuredData = !$opt->disable_rfc_sdata;
 $Parse::Syslog::Line::RFC5424StructuredDataStrict = $opt->strict_rfc_sdata;
 
@@ -80,7 +80,7 @@ parse-syslog-line.pl - Parse a syslog message and display the structured data
 
 =head1 VERSION
 
-version 6.0
+version 6.1
 
 =head1 SYNOPSIS
 
